local planet = ents.Derive("base")
planet.selectable = true


function planet:load(x, y)
	self:setPos( x, y )
	self.resources = {}
	
	self.productivity = 100 -- Ship production rate
	self.power = 10000 -- Amount of starting ships
	self.lastSpawn = 0
end

function planet:setRadius(rad)
	self.radius = rad
end

function planet:setPlanetType(pType)
	--TODO: Make a garuntee it is a valid type!
	self.pType = pType
end

function planet:draw()
	if not( self.isAlive and self:isVisible() )then return end
	
	love.graphics.setColor( 255, 255, 255, 255 )
	local sx,sy = self:getSize()
	
	local radScale = clamp(self.radius * zoomScale, 8, 180)
	love.graphics.setColor( 198, 241, 255, 255 )       -- NEGATE Y!
	love.graphics.circle("fill",  self.x * zoomScale,  -self.y * zoomScale, self.radius * zoomScale, radScale)
	
	
	--Draw Info if we are hovered over it
	
	if (distance(getWorldX(),getWorldY(),self.x,self.y) <= self.radius) then -- Wonder if it will be worthwhile to save worldX and worldY in a variable and update it instead?
		love.graphics.setColor( 255, 0, 0, 255 )
		love.graphics.setLineWidth( 1 )
		love.graphics.setLineStyle("rough")
		love.graphics.rectangle( "line", (self.x - sx - PLAN_PAD)*zoomScale, (-self.y - sy - PLAN_PAD )*zoomScale, (2*PLAN_PAD + sx * 2)*zoomScale, (2*PLAN_PAD + sy * 2)*zoomScale )
	
		-- We are on CANVAS now, so Y = -Y
		local tooltipX = (self.x + self.radius + PLAN_PAD )*zoomScale
		local tooltipY = (-self.y - self.radius - PLAN_PAD )*zoomScale
		love.graphics.print( "Planet: " .. self.name, tooltipX, tooltipY, 0, 1, 1 )
		love.graphics.print( "Production Rate: " .. self.productivity .. "%", tooltipX, tooltipY + 14, 0, 1, 1 )
		love.graphics.print( "Power: " .. self.power, tooltipX, tooltipY + 28 , 0, 1, 1 )
		
		love.graphics.print( "Resources:", tooltipX, tooltipY + 50 , 0, 1, 1 )
		if #self.resources == 0 then
			love.graphics.print( "No resources available", tooltipX, tooltipY + 64 , 0, 1, 1 )
		else
			local offset = 64
			for k, v in pairs(self.resources) do
				love.graphics.print( v.name ..": " .. v.amount, tooltipX, tooltipY + offset , 0, 1, 1 )
				offset = offset + 12
			end
		end
	end
	
end	

function planet:spawn()
	self.isAlive = true
	self.radius = self.radius or math.random(3,10)
	
	--Fill up our resources
	
	--Determine our color (Or texture in the future)
	
	-- Get a name or generate one
	self.name = self.name or "PLANET " .. self.id
	self.temperature = self.temperature or 78
	self.pType = self.pType or "Unknown"
	self.resources = {{name = "Iron", amount = 100},{name = "Copper", amount = 100},{name = "Coal", amount = 100},{name = "Soil", amount = 100},{name = "Water", amount = 100}}
	
	-- make some cities?
end

function planet:getSize()
	return self.radius, self.radius
end

function planet:spawnShip(targetPlanet)
	local ship = ents.Create("ship",self.x,self.y)
	ship:setTarget(targetPlanet)
	ship:spawn()
	table.insert(game.ships, ship)
end

function planet:queueShips(targetPlanet, samount)
	if not self.queue then
		self.queue = {}
	end
	
	table.insert(self.queue, {target = targetPlanet, amount = samount})
end

function planet:update(dt)
	self.lastSpawn = self.lastSpawn + dt
	
	--Update launche queues
	if self.queue then
		local launch = self.queue[1]
		
		if (self.power > 0) and self.lastSpawn >= SHIP_LAUNCH_TIME * ( 100 / self.productivity ) then
			self:spawnShip(launch.target)
			
			self.lastSpawn = 0
			self.power = self.power - 1
			launch.amount = launch.amount - 1
			
			if launch.amount == 0 then
				table.remove(self.queue, 1)
			end
			
			if #self.queue == 0 then
				self.queue = nil
			end
		end
	end
	
	-- Update orbits?
end

return planet
